@@ -26,6 +26,7 @@ except ImportError:
     def install_pytest_asserts():
         pass
 
+from toolium.utils import dataset
 from toolium.config_files import ConfigFiles
 from toolium.driver_wrapper import DriverWrappersPool
 from toolium.jira import add_jira_status, change_all_jira_status, save_jira_conf
@@ -65,6 +66,10 @@ def before_all(context):
     # Behave dynamic environment
     context.dyn_env = DynamicEnvironment(logger=context.logger)
 
+    # Initialize dataset behave variables
+    dataset.behave_context = context
+    dataset.toolium_config = context.toolium_config
+
 
 def before_feature(context, feature):
     """Feature initialization
@@ -271,12 +276,13 @@ def after_feature(context, feature):
     :param context: behave context
     :param feature: running feature
     """
-    # Behave dynamic environment
-    context.dyn_env.execute_after_feature_steps(context)
-
-    # Close drivers
-    DriverWrappersPool.close_drivers(scope='module', test_name=feature.name,
-                                     test_passed=context.global_status['test_passed'])
+    try:
+        # Behave dynamic environment
+        context.dyn_env.execute_after_feature_steps(context)
+    finally:
+        # Close drivers regardless of an Exception being raised due to failed preconditions
+        DriverWrappersPool.close_drivers(scope='module', test_name=feature.name,
+                                         test_passed=context.global_status['test_passed'])
 
 
 def after_all(context):
