@@ -86,7 +86,7 @@ class PageElement(CommonObject):
         try:
             self._find_web_element()
         except NoSuchElementException as exception:
-            parent_msg = " and parent locator '{}'".format(self.parent) if self.parent else ''
+            parent_msg = f" and parent locator {self.parent_locator_str()}" if self.parent else ''
             msg = "Page element of type '%s' with locator %s%s not found"
             self.logger.error(msg, type(self).__name__, self.locator, parent_msg)
             exception.msg += "\n  {}".format(msg % (type(self).__name__, self.locator, parent_msg))
@@ -134,7 +134,7 @@ class PageElement(CommonObject):
                 context_dict = next(
                     (item for item in contexts if 'webviewName' in item and item['webviewName'] == app_web_context),
                     None)
-                if context_dict:
+                if context_dict and 'pages' in context_dict:
                     context = app_web_context
                     window_handle = 'CDwindow-{}'.format(context_dict['pages'][0]['id'])
             if context:
@@ -169,6 +169,19 @@ class PageElement(CommonObject):
             if self.driver.context != PageElement.native_context:
                 self.driver.switch_to.context(PageElement.native_context)
 
+    def parent_locator_str(self):
+        """Return string with locator tuple for parent element
+
+        :returns: parent element locator
+        """
+        if isinstance(self.parent, PageElement):
+            parent_locator = str(self.parent.locator)
+        elif isinstance(self.parent, tuple):
+            parent_locator = str(self.parent)
+        else:
+            parent_locator = '[WebElement without locator]'
+        return parent_locator
+
     def scroll_element_into_view(self):
         """Scroll element into view
 
@@ -218,7 +231,7 @@ class PageElement(CommonObject):
                 condition_msg = 'not found or is not clickable'
                 self.utils.wait_until_element_clickable(self, timeout)
         except TimeoutException as exception:
-            parent_msg = " and parent locator '{}'".format(self.parent) if self.parent else ''
+            parent_msg = f" and parent locator {self.parent_locator_str()}" if self.parent else ''
             timeout = timeout if timeout else self.utils.get_explicitly_wait()
             msg = "Page element of type '%s' with locator %s%s %s after %s seconds"
             msg = msg % (type(self).__name__, self.locator, parent_msg, condition_msg, timeout)
